@@ -48,12 +48,16 @@ SOFTWARE.
 ******************************************************************/
 /* $XConsortium: afbgc.c,v 5.35 94/04/17 20:28:23 dpw Exp $ */
 
-#include "X.h"
-#include "Xmd.h"
-#include "Xproto.h"
+#ifdef HAVE_DIX_CONFIG_H
+#include <dix-config.h>
+#endif
+
+#include <X11/X.h>
+#include <X11/Xmd.h>
+#include <X11/Xproto.h>
 #include "afb.h"
 #include "dixfontstr.h"
-#include "fontstruct.h"
+#include <X11/fonts/fontstruct.h>
 #include "gcstruct.h"
 #include "windowstr.h"
 #include "pixmapstr.h"
