@@ -1,9 +1,5 @@
 /* $XFree86: xc/programs/Xserver/hw/xfree86/rac/xf86RAC.h,v 1.2 1999/05/15 12:10:33 dawes Exp $ */
 
-#ifdef HAVE_XORG_CONFIG_H
-#include <xorg-config.h>
-#endif
-
 #ifndef __XF86RAC_H
 #define __XF86RAC_H 1
 
