@@ -49,6 +49,10 @@ SOFTWARE.
 /* GJA -- modified this file for vga16 */
 /* $XConsortium: mfbline.c /main/4 1996/02/21 17:56:48 kaleb $ */
 
+#ifdef HAVE_XORG_CONFIG_H
+#include <xorg-config.h>
+#endif
+
 #include "xf4bpp.h"
 #include "OScompiler.h"
 #include "mfbmap.h"
